import datetime
from django.template.defaultfilters import date
from django.contrib.humanize.templatetags.humanize import naturalday

try:
    from django.contrib.humanize.templatetags.humanize import naturaltime
except ImportError:
    from .compat.humanize import naturaltime

DEFAULT_TIMESTAMP_FIELDS = ('created', 'modified')


def human_timestamp(value):
    '''
    Converts the given date(time) value to a more readable string.

    If the delta between now and the given value is < 1 day it
    will return strings like 'a second ago', '2 hours ago' etc.

    Otherwise the datetime will be formatted according to the given
    format or ``SHORT_DATE(TIME)_FORMAT``
    '''
    today = datetime.date.today()
    if type(value) is datetime.date:
        delta = today - value
        fmt = 'SHORT_DATE_FORMAT'
        display_method = naturalday
        delta_range = [-1, 0, 1]
    else:
        delta = today - value.date()
        fmt = 'SHORT_DATETIME_FORMAT'
        display_method = naturaltime
        delta_range = [0]
    if delta.days in delta_range:
        return display_method(value)
    return date(value, fmt)


class TimestampedChangelist(object):
    '''
    Custom changelist implementation that adds human readable
    timestamp fields to the list display.
    '''

    def __init__(self, *args, **kwargs):
        super(TimestampedChangelist, self).__init__(*args, **kwargs)
        self.timestamp_display_methods = []
        for field in self.model_admin.timestamp_fields:
            self.timestamp_display_methods.append(
                self.create_list_display(field))
        self.add_timestamps_to_list_display()

    def add_timestamps_to_list_display(self):
        '''
        Appends the created and modified fields to the list display, unless
        they are already in there.
        '''
        for method in self.timestamp_display_methods:
            self.list_display.append(method)

    def get_timestamp_verbose_name(self, field):
        '''
        Get the accessor name as configured using the model admin's
        ``timestamp_fields`` attribute.
        '''
        return self.model._meta.get_field(field).verbose_name

    def get_timestamp(self, obj, field):
        '''
        Get either the created or modified datetime from the model.
        '''
        if hasattr(obj, field):
            return getattr(obj, field)

    def create_list_display(self, field):
        '''
        Create a timestamp list_display method.
        '''
        method = lambda obj: human_timestamp(
                                self.get_timestamp(obj, field))
        method.admin_order_field = field
        method.short_description = self.get_timestamp_verbose_name(field)
        return method


class TimestampedAdminMixin(object):
    '''
    This mixin adds custom display of timestamp fields to the Django
    admin changelist.
    '''

    timestamp_fields = DEFAULT_TIMESTAMP_FIELDS

    def get_changelist(self, request, **kwargs):
        '''
        Hands back the custom changelist implementation.
        '''
        ChangeList = super(TimestampedAdminMixin, self)\
                       .get_changelist(request, **kwargs)
        return type('%s$TimestampedChangelist' % self.__class__.__name__,
                    (TimestampedChangelist, ChangeList), {})



from os.path import dirname, join

DEBUG = True
TEMPLATE_DEBUG = DEBUG

DATABASE_ENGINE = 'sqlite3'
DATABASE_NAME = join(dirname(__file__), 'db', 'admintimestamps.db')

DATABASES = {
    "default": {
        "ENGINE": 'django.db.backends.sqlite3',
        "NAME": join(dirname(__file__), 'db', 'admintimestamps.db')
    }
}

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'admintimestamps',
    'testproject',
)

ROOT_URLCONF = 'testproject.urls'



import os.path
import sys

from django.conf import settings
from django.core import management


def runtests():
    sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))
    settings_mod = getattr(__import__('testproject.settings'), 'settings')
    management.setup_environ(settings_mod)
    from django.test.utils import get_runner
    try:
        from django.test.simple import DjangoTestSuiteRunner
        run_tests = get_runner(settings)().run_tests
    except ImportError:
        # for Django versions that don't have DjangoTestSuiteRunner
        run_tests = get_runner(settings)
    failures = run_tests(['testproject'], verbosity=1, interactive=True)
    sys.exit(failures)


if __name__ == '__main__':
    runtests()



import re

from admin import TimestampedAdmin
from admintimestamps.base import TimestampedChangelist
from datetime import datetime, timedelta
from django import test
from django.contrib.auth.models import User
from models import AlternativeTimestampedItem, FakeTimestampedItem, \
    SingleTimestampedItem, DateStampedItem


class TestAdmin(test.TestCase):

    def setUp(self):
        self.admin = TimestampedAdmin(FakeTimestampedItem, None)

    def test_get_changelist(self):
        self.assertTrue(issubclass(
            self.admin.get_changelist(None), TimestampedChangelist),
            'Expected changelist to be a subclass of TimestampedChangelist')

    def test_has_timestamp_fields(self):
        self.assertTrue(hasattr(self.admin, 'timestamp_fields'),
                        'Expected timestamp_fields on ModelAdmin')


class TestChangeList(test.TestCase):

    # Workaround for Django 1.2
    if not hasattr(test.TestCase, 'assertRegexpMatches'):
        def assertRegexpMatches(self, text, expected_regexp, msg=None):
            '''Fail the test unless the text matches the regular expression.'''
            if isinstance(expected_regexp, basestring):
                expected_regexp = re.compile(expected_regexp)
            if not expected_regexp.search(text):
                msg = msg or 'Regexp didn\'t match'
                msg = '%s: %r not found in %r' % (msg,
                                                  expected_regexp.pattern,
                                                  text)
                raise self.failureException(msg)

    def setUp(self):
        FakeTimestampedItem(title='just now',
                            created=datetime.now() - timedelta(minutes=1),
                            modified=datetime.now() - timedelta(minutes=2)
                           ).save()
        FakeTimestampedItem(title='some time ago',
                            created=datetime(2010, 1, 15, 21, 19),
                            modified=datetime(2011, 5, 28, 9, 59)).save()
        AlternativeTimestampedItem(title='alternative').save()
        SingleTimestampedItem(title='single timestamp').save()
        DateStampedItem(title='datestamp').save()
        user = User.objects.create_user('test', 'test@example.com', 'test')
        user.is_staff = user.is_superuser = True
        user.save()
        self.client.login(username='test', password='test')

    def test_header_row(self):
        response = self.client.get('/admin/testproject/faketimestampeditem/')
        self.assertRegexpMatches(response.content,
                                 r'Created\s*</a></(?:th|div)>')
        self.assertRegexpMatches(response.content,
                                 r'Modified\s*</a></(?:th|div)>')

    def test_recent_cell_values(self):
        response = self.client.get('/admin/testproject/faketimestampeditem/')
        self.assertContains(response, '<td>a minute ago', count=1)
        self.assertContains(response, '<td>2 minutes ago', count=1)

    def test_some_time_ago_values(self):
        response = self.client.get('/admin/testproject/faketimestampeditem/')
        self.assertContains(response, '<td>01/15/2010 9:19 p.m.</td>',
                            count=1)
        self.assertContains(response, '<td>05/28/2011 9:59 a.m.</td>',
                            count=1)

    def test_alternative_header_row(self):
        response = self.client.get(
                     '/admin/testproject/alternativetimestampeditem/')
        self.assertRegexpMatches(response.content,
                                 r'Created at\s*</a></(?:th|div)>')
        self.assertRegexpMatches(response.content,
                                 r'Modified at\s*</a></(?:th|div)>')

    def test_single_header_row(self):
        response = self.client.get(
                     '/admin/testproject/singletimestampeditem/')
        self.assertRegexpMatches(response.content,
                                 r'Modified\s*</a></(?:th|div)>')

    def test_datestamp_header_row(self):
        response = self.client.get(
                     '/admin/testproject/datestampeditem/')
        self.assertRegexpMatches(response.content,
                                 r'Created\s*</a></(?:th|div)>')
        self.assertRegexpMatches(response.content,
                                 r'Modified\s*</a></(?:th|div)>')



from django.contrib import admin
from models import TimestampedItem, FakeTimestampedItem,\
    AlternativeTimestampedItem, SingleTimestampedItem, DateStampedItem
from admintimestamps import TimestampedAdminMixin


class TimestampedAdmin(TimestampedAdminMixin, admin.ModelAdmin):
    pass

admin.site.register(TimestampedItem, TimestampedAdmin)
admin.site.register(FakeTimestampedItem, TimestampedAdmin)


class SingleTimestampedAdmin(TimestampedAdminMixin, admin.ModelAdmin):
    timestamp_fields = ('modified',)

admin.site.register(SingleTimestampedItem, SingleTimestampedAdmin)


class DateStampedAdmin(TimestampedAdminMixin, admin.ModelAdmin):
    pass

admin.site.register(DateStampedItem, DateStampedAdmin)


class AlternativeTimestampedAdmin(TimestampedAdmin):
    timestamp_fields = ('created_at', 'modified_at')

admin.site.register(AlternativeTimestampedItem, AlternativeTimestampedAdmin)



import codecs
from os.path import join, dirname
from setuptools import setup

version = '1.1'
read = lambda *rnames: unicode(codecs.open(join(dirname(__file__), *rnames),
                                           encoding='utf-8').read()
                              ).strip()


setup(
    name='django-admin-timestamps',
    version=version,
    description='Custom list display of model timestamps for Django Admin.',
    long_description='\n\n'.join((read('README'), read('CHANGES'),)),
    author='Jaap Roes',
    author_email='jaap.roes@gmail.com',
    url='https://bitbucket.org/jaap3/django-admin-timestamps/',
    package_dir={'': 'src'},
    packages=['admintimestamps', 'admintimestamps.compat'],
    license='MIT',
    classifiers=[
      'Development Status :: 5 - Production/Stable',
      'Environment :: Web Environment',
      'Framework :: Django',
      'Intended Audience :: Developers',
      'License :: OSI Approved :: MIT License',
      'Operating System :: OS Independent',
      'Programming Language :: Python',
    ],
    tests_require=['Django>=1.2', 'unittest2>=0.5.1'],
    test_suite='tests.runtests.runtests',
    zip_safe=False,
)



from django.db import models


class TimestampedItem(models.Model):
    title = models.CharField('title', max_length=100)
    created = models.DateTimeField('created', auto_now_add=True)
    modified = models.DateTimeField('modified', auto_now=True)

    def __unicode__(self):
        return self.title


class SingleTimestampedItem(models.Model):
    title = models.CharField('title', max_length=100)
    modified = models.DateTimeField('modified', auto_now=True)

    def __unicode__(self):
        return self.title


class DateStampedItem(models.Model):
    title = models.CharField('title', max_length=100)
    created = models.DateField('created', auto_now_add=True)
    modified = models.DateTimeField('modified', auto_now=True)

    def __unicode__(self):
        return self.title


class FakeTimestampedItem(models.Model):
    '''
    This model doesn't actually set it's timestamps, makes it easier
    to get predictable and testable formatted output.
    '''
    title = models.CharField('title', max_length=100)
    created = models.DateTimeField('created')
    modified = models.DateTimeField('modified')

    def __unicode__(self):
        return self.title


class AlternativeTimestampedItem(models.Model):
    title = models.CharField('title', max_length=100)
    created_at = models.DateTimeField(auto_now_add=True)
    modified_at = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return self.title


'''
Taken from Django's trunk's (pre-1.4)
``django.contrib.humanize.templatetags.humanize``
'''

from datetime import datetime, timedelta

from django.template import defaultfilters
from django.utils.translation import ungettext, ugettext as _
from django.utils.tzinfo import LocalTimezone


# XXX: References to pgettext removed to be Django 1.2 compatible.

def naturaltime(value):
    """
    For date and time values shows how many seconds, minutes or hours ago
    compared to current timestamp returns representing string.
    """
    try:
        value = datetime(value.year, value.month, value.day, value.hour,
                         value.minute, value.second)
    except AttributeError:
        return value
    except ValueError:
        return value

    if getattr(value, 'tzinfo', None):
        now = datetime.now(LocalTimezone(value))
    else:
        now = datetime.now()
    now = now - timedelta(0, 0, now.microsecond)
    if value < now:
        delta = now - value
        if delta.days != 0:
            return _(
                '%(delta)s ago'
            ) % {'delta': defaultfilters.timesince(value)}
        elif delta.seconds == 0:
            return _(u'now')
        elif delta.seconds < 60:
            return ungettext(
                u'a second ago', u'%(count)s seconds ago', delta.seconds
            ) % {'count': delta.seconds}
        elif delta.seconds // 60 < 60:
            count = delta.seconds // 60
            return ungettext(
                u'a minute ago', u'%(count)s minutes ago', count
            ) % {'count': count}
        else:
            count = delta.seconds // 60 // 60
            return ungettext(
                u'an hour ago', u'%(count)s hours ago', count
            ) % {'count': count}
    else:
        delta = value - now
        if delta.days != 0:
            return _(
                '%(delta)s from now'
            ) % {'delta': defaultfilters.timeuntil(value)}
        elif delta.seconds == 0:
            return _(u'now')
        elif delta.seconds < 60:
            return ungettext(
                u'a second from now', u'%(count)s seconds from now',
                delta.seconds
            ) % {'count': delta.seconds}
        elif delta.seconds // 60 < 60:
            count = delta.seconds // 60
            return ungettext(
                u'a minute from now', u'%(count)s minutes from now', count
            ) % {'count': count}
        else:
            count = delta.seconds // 60 // 60
            return ungettext(
                u'an hour from now', u'%(count)s hours from now', count
            ) % {'count': count}
